@@ -435,7 +435,7 @@ void read_pwsafe_file(MainView* mainview) {
 	g_assert(mainview!=NULL);
 	if (!mainview->file_name) return;
 	file_utf8=g_filename_to_utf8(mainview->file_name, -1, NULL, NULL, NULL);
-	mainview->pass=get_password(mainview, file_utf8);
+	mainview->pass=get_password(mainview, file_utf8, PWD_DLG_GET);
 	g_free(file_utf8);
 	
 	if (mainview->pass && mainview->pass[0]) {
@@ -483,9 +483,9 @@ void read_pwsafe_file(MainView* mainview) {
 									fileversion=buffer;
 									if (strcmp("2.0", fileversion)!=0) {
 										v2 = FALSE;
-//										msg = g_strdup(_("Reading file with version (1.x) \n Will write in 2.0 format"));
-//										msgbox(mainview, msg);
-//										g_free(msg);
+										msg = g_strdup(_("Reading file with version (1.x) \n Will write in 2.0 format"));
+										msgbox(mainview, msg);
+										g_free(msg);
 										g_printerr("Reading file with version (1.x) \n Will write in 2.0 format");
 										entry=g_malloc0(sizeof(rec));
 										entry->uuid=entry->group=entry->title=entry->user=entry->notes=entry->password=NULL;
