@@ -100,7 +100,7 @@ MainView* interface_main_view_new( AppData *data ) {
 	DBusConnection *con = osso_get_sys_dbus_connection(data->osso);
 	/* Add the callback, which should be called, once the device is rotated */
 	dbus_bus_add_match(con, MCE_MATCH_RULE, NULL);
-	dbus_connection_add_filter(con, dbus_handle_mce_message, result, NULL);
+	dbus_connection_add_filter(con, (DBusHandleMessageFunction *) dbus_handle_mce_message, result, NULL);
 
 #if MAEMO_VERSION_MAJOR >= 5
 	result->regexHttp = g_regex_new("http(s?)://[\\S]+", G_REGEX_CASELESS, 0, NULL);
@@ -124,14 +124,13 @@ GtkWidget *show_rec_line(GtkWidget *sizer, char *label, char *txt, int row, int
 #if MAEMO_VERSION_MAJOR < 5
 	gtk_table_attach_defaults(GTK_TABLE(sizer), gtk_label_new(label), col, col+1, row, row+1);
 	textentry=gtk_entry_new();
-	if (txt!=NULL) gtk_entry_set_text(GTK_ENTRY(textentry), txt);
 	gtk_table_attach_defaults(GTK_TABLE(sizer), textentry, col+1,col+2, row, row+1);
 #else
 	textentry=hildon_entry_new(HILDON_SIZE_AUTO);
 	hildon_entry_set_placeholder(HILDON_ENTRY(textentry), label);
-	if (txt!=NULL) hildon_entry_set_text(HILDON_ENTRY(textentry), txt);
 	gtk_table_attach_defaults(GTK_TABLE(sizer), textentry, col,col+2, row, row+1);
 #endif
+	if (txt!=NULL) gtk_entry_set_text(GTK_ENTRY(textentry), txt);
 	return textentry;
 }
 
@@ -143,20 +142,95 @@ void unshow_rec(MainView *main) {
 	}
 }
 
-gchar *get_password(MainView *main, gchar *caption) {
+gchar *get_password(MainView *main, gchar *caption, gint pwdDlgType) {
+	PasswordUI *pwUi;
 	gchar *result=NULL;
-	GtkWidget*  pwdialog;
-	pwdialog = hildon_get_password_dialog_new(GTK_WINDOW(main->data->main_view), FALSE);
-#if HILDON == 1
-	hildon_get_password_dialog_set_message (HILDON_GET_PASSWORD_DIALOG(pwdialog), caption);
+#if MAEMO_VERSION_MAJOR >= 5
+	GtkWidget *image;
+#endif
+	GtkWidget *contentArea;
+	GtkWidget *actionArea;
+
+	pwUi = g_new0(PasswordUI, 1);
+	pwUi->type = pwdDlgType;
+	pwUi->main = main;
+	pwUi->result = NULL;
+	pwUi->msg = NULL;
+	
+	pwUi->dialog = gtk_dialog_new_with_buttons(caption, GTK_WINDOW(main->data->main_view), GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR, NULL);
+#if MAEMO_VERSION_MAJOR < 5
+	actionArea = GTK_DIALOG(pwUi->dialog)->action_area;
+	contentArea = GTK_DIALOG(pwUi->dialog)->vbox;
+
+	pwUi->button = gtk_button_new_from_stock(GTK_STOCK_OK);
+	pwUi->entry1=gtk_entry_new();
 #else
-	g_object_set(pwdialog, "domain", caption, NULL);
+	actionArea = gtk_dialog_get_action_area(GTK_DIALOG(pwUi->dialog));
+	contentArea = gtk_dialog_get_content_area(GTK_DIALOG(pwUi->dialog));
+
+	pwUi->button =  hildon_button_new_with_text(HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT, HILDON_BUTTON_ARRANGEMENT_VERTICAL, _("OK"), NULL);
+	image = gtk_image_new_from_stock (GTK_STOCK_OK, GTK_ICON_SIZE_BUTTON);
+	hildon_button_set_image (HILDON_BUTTON (pwUi->button), image);
+	pwUi->entry1=hildon_entry_new(HILDON_SIZE_AUTO);
+#endif
+	gtk_container_add (GTK_CONTAINER (actionArea), pwUi->button);
+	gtk_container_add (GTK_CONTAINER (contentArea), pwUi->entry1);
+
+	gtk_widget_set_sensitive(pwUi->button, FALSE);
+	g_signal_connect (pwUi->button, "clicked", G_CALLBACK (callback_pwdialog_button), pwUi);
+
+	gtk_entry_set_visibility(GTK_ENTRY(pwUi->entry1), FALSE);
+	g_signal_connect(G_OBJECT(pwUi->entry1), "changed", G_CALLBACK(callback_pwdialog_changed), pwUi);
+	g_signal_connect(G_OBJECT(pwUi->entry1), "activate", G_CALLBACK(callback_pwdialog_activate), pwUi);
+	if (pwdDlgType != PWD_DLG_GET) {
+#if MAEMO_VERSION_MAJOR < 5
+		pwUi->entry2=gtk_entry_new();
+#else
+		pwUi->entry2=hildon_entry_new(HILDON_SIZE_AUTO);
+#endif
+		gtk_entry_set_visibility(GTK_ENTRY(pwUi->entry2), FALSE);
+		gtk_container_add (GTK_CONTAINER (contentArea), pwUi->entry2);
+		g_signal_connect(G_OBJECT(pwUi->entry2), "changed", G_CALLBACK(callback_pwdialog_changed), pwUi);
+		g_signal_connect(G_OBJECT(pwUi->entry2), "activate", G_CALLBACK(callback_pwdialog_activate), pwUi);
+		if (pwdDlgType != PWD_DLG_NEW) {
+#if MAEMO_VERSION_MAJOR < 5
+			pwUi->entry3=gtk_entry_new();
+#else
+			pwUi->entry3=hildon_entry_new(HILDON_SIZE_AUTO);
+#endif
+			gtk_entry_set_visibility(GTK_ENTRY(pwUi->entry3), FALSE);
+#if MAEMO_VERSION_MAJOR < 5
+#else
+			hildon_entry_set_placeholder(HILDON_ENTRY(pwUi->entry1), _("Old password"));
+			hildon_entry_set_placeholder(HILDON_ENTRY(pwUi->entry2), _("New password"));
+			hildon_entry_set_placeholder(HILDON_ENTRY(pwUi->entry3), "Confirm new password");
+#endif
+			gtk_container_add (GTK_CONTAINER (contentArea), pwUi->entry3);
+			g_signal_connect(G_OBJECT(pwUi->entry3), "changed", G_CALLBACK(callback_pwdialog_changed), pwUi);
+			g_signal_connect(G_OBJECT(pwUi->entry3), "activate", G_CALLBACK(callback_pwdialog_activate), pwUi);
+		} else {
+#if MAEMO_VERSION_MAJOR < 5
+#else
+			hildon_entry_set_placeholder(HILDON_ENTRY(pwUi->entry1), _("Password"));
+			hildon_entry_set_placeholder(HILDON_ENTRY(pwUi->entry2), _("Confirm password"));
+#endif
+		}
+	} else {
+#if MAEMO_VERSION_MAJOR < 5
+#else
+		hildon_entry_set_placeholder(HILDON_ENTRY(pwUi->entry1), _("Password"));
 #endif
-	gtk_widget_show(GTK_WIDGET(pwdialog));
-	if (GTK_RESPONSE_OK==gtk_dialog_run(GTK_DIALOG(pwdialog))) {
-		result=g_strdup(hildon_get_password_dialog_get_password(HILDON_GET_PASSWORD_DIALOG(pwdialog)));
 	}
-	gtk_widget_destroy (GTK_WIDGET (pwdialog));
+	gtk_widget_show_all(GTK_WIDGET(pwUi->dialog));
+	gtk_dialog_run(GTK_DIALOG(pwUi->dialog));
+	
+	// Getting out of the password Dialog
+	if (pwUi->result != NULL) {
+		result = g_strdup(pwUi->result);
+		g_free(pwUi->result);
+	}
+	g_free(pwUi);
+	gtk_widget_destroy (GTK_WIDGET (pwUi->dialog));
 	return result;
 }
 
@@ -216,7 +290,7 @@ void show_rec(MainView *main, rec *r) {
 	selector=hildon_touch_selector_entry_new_text();
 	hildon_entry_set_placeholder(hildon_touch_selector_entry_get_entry(HILDON_TOUCH_SELECTOR_ENTRY(selector)), _("Group"));
 	if (r->group!=NULL) {
-		hildon_entry_set_text(hildon_touch_selector_entry_get_entry(HILDON_TOUCH_SELECTOR_ENTRY(selector)), r->group);
+		gtk_entry_set_text(GTK_ENTRY(hildon_touch_selector_entry_get_entry(HILDON_TOUCH_SELECTOR_ENTRY(selector))), r->group);
 	}
 	hildon_picker_button_set_selector (HILDON_PICKER_BUTTON (rec_win->group), HILDON_TOUCH_SELECTOR (selector));
 
@@ -540,6 +614,9 @@ void interface_show_state(MainView *main) {
 		gtk_widget_set_sensitive(GTK_WIDGET(main->change_password_tb), fopen);
 		gtk_widget_set_sensitive(GTK_WIDGET(main->add_tb), fopen);
 		gtk_editable_set_editable(GTK_EDITABLE(main->filter_tb), fopen);
+		if (!fopen) {
+			gtk_entry_set_text(GTK_ENTRY(main->filter_tb), "");
+		}
 	}
 }
 
